/*
 * Copyright (c) 2006-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: MyProxyClient.java,v 1.9 2007/02/21 01:32:04 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.myproxy;

import java.io.IOException;
import java.security.GeneralSecurityException;
import java.security.cert.X509Certificate;

import org.globus.gsi.TrustedCertificates;
import org.globus.myproxy.MyProxy;
import org.globus.myproxy.MyProxyException;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;

import edu.indiana.extreme.xbaya.security.XBayaSecurity;

import xsul5.MLogger;

/**
 * @author Satoshi Shirasuna
 */
public class MyProxyClient {

    private static final MLogger logger = MLogger.getLogger();

    private boolean initialized = false;

    private X509Certificate[] trustedCertificates;

    private String server;

    private int port;

    private String username;

    private String passphrase;

    private int lifetime;

    private MyProxy myproxy;

    private GSSCredential proxy;

    /**
     * Constructs a MyProxyClient.
     */
    public MyProxyClient() {
        // Nothing
    }

    /**
     * Constructs a MyProxyClient.
     * 
     * @param myproxyServer
     * @param myproxyPort
     * @param myproxyUsername
     * @param myproxyPassphrase
     * @param myproxyLifetime
     */
    public MyProxyClient(String myproxyServer, int myproxyPort,
            String myproxyUsername, String myproxyPassphrase,
            int myproxyLifetime) {
        set(myproxyServer, myproxyPort, myproxyUsername, myproxyPassphrase,
                myproxyLifetime);
    }

    /**
     * @return The array of trusted certificates.
     */
    public X509Certificate[] getTrustedCertificates() {
        if (!this.initialized) {
            init();
        }
        return this.trustedCertificates;
    }

    /**
     * @param myproxyServer
     * @param myproxyPort
     * @param myproxyUsername
     * @param myproxyPassphrase
     * @param myproxyLifetime
     */
    public void set(String myproxyServer, int myproxyPort,
            String myproxyUsername, String myproxyPassphrase,
            int myproxyLifetime) {
        this.server = myproxyServer;
        this.port = myproxyPort;
        this.username = myproxyUsername;
        this.passphrase = myproxyPassphrase;
        this.lifetime = myproxyLifetime;
    }

    /**
     * Returns the lifetime.
     * 
     * @return The lifetime
     */
    public int getLifetime() {
        return this.lifetime;
    }

    /**
     * Returns the password.
     * 
     * @return The password
     */
    public String getPassphrase() {
        return this.passphrase;
    }

    /**
     * Returns the port.
     * 
     * @return The port
     */
    public int getPort() {
        return this.port;
    }

    /**
     * Returns the server.
     * 
     * @return The server
     */
    public String getServer() {
        return this.server;
    }

    /**
     * Returns the username.
     * 
     * @return The username
     */
    public String getUsername() {
        return this.username;
    }

    /**
     * @return The proxy. null if it's not loaded.
     */
    public GSSCredential getProxy() {
        return this.proxy;
    }

    /**
     * @param myproxyServer
     * @param myproxyPort
     * @param myproxyUsername
     * @param myproxyPassphrase
     * @param myproxyLifetime
     * @throws MyProxyException
     * @throws IOException
     * @throws GeneralSecurityException
     */
    public void load(String myproxyServer, int myproxyPort,
            String myproxyUsername, String myproxyPassphrase,
            int myproxyLifetime) throws MyProxyException {
        set(myproxyServer, myproxyPort, myproxyUsername, myproxyPassphrase,
                myproxyLifetime);
        load();
    }

    /**
     * @throws MyProxyException
     * @throws IOException
     * @throws GeneralSecurityException
     */
    public void load() throws MyProxyException {
        if (this.server == null || this.server.length() == 0) {
            // Should not happen
            throw new IllegalStateException();
        }
        if (this.port == 0) {
            // Should not happen
            throw new IllegalStateException();
        }
        if (this.username == null || this.username.length() == 0) {
            // Should not happen
            throw new IllegalStateException();
        }
        if (this.passphrase == null || this.passphrase.length() == 0) {
            // Should not happen
            throw new IllegalStateException();
        }
        if (!this.initialized) {
            init();
        }

        this.myproxy = new MyProxy(this.server, this.port);
        this.proxy = this.myproxy.get(this.username, this.passphrase,
                this.lifetime);
    }

    /**
     * @return true if a proxy has been loaded and has enough lifetime ramained;
     *         false otherwise.
     */
    public boolean isProxyValid() {
        if (this.proxy == null) {
            return false;
        } else {
            try {
                int remainingLifetime = this.proxy.getRemainingLifetime();
                if (remainingLifetime > 60 * 10 /* 10 mins */) {
                    // Good case.
                    return true;
                } else {
                    return false;
                }
            } catch (GSSException e) {
                logger.caught(e);
                return false;
            }
        }
    }

    private void init() {
        // Load CA certificates from a file included in the XBaya jar.
        this.trustedCertificates = XBayaSecurity.getTrustedCertificates();

        TrustedCertificates certificatesArray = new TrustedCertificates(
                this.trustedCertificates);
        TrustedCertificates.setDefaultTrustedCertificates(certificatesArray);
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2006-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
